# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301,C0103,W0614,W0401,E0202

# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Helper functions for dealing with OpenURL, refs #5163.
"""

from six.moves.urllib.parse import urlencode

def update_on_value(t, tkey, value, first=True):
    """
    Update dictionary t and set t[tkey] to s[skey] exists and is not None. If
    the value in question is a sequence, first controls, whether only the first
    element should be taken.
    """
    if value is not None:
        if first and isinstance(value, (list, tuple)):
            if len(value) > 0:
                t[tkey] = value[0]
        else:
            t[tkey] = value

def openurl_parameters_from_intermediateschema(doc, rfr_id='www.ub.uni-leipzig.de'):
    """
    Given a intermediate schema dictionary, generate OpenURL parameters to be
    used with a (REDI) link resolver.

    Roughly like vufind13/trunk/web/RecordDrivers/AIRecord.php #L1032-1336.
    """
    if not rfr_id:
        raise ValueError('rfr_id must be set')

    if not doc:
        return {}

    params = {
        'url_ver': 'Z39.88-2004',
        'ctx_ver': 'Z39.88-2004',
        'ctx_enc': 'info:ofi/enc:UTF-8',
        'rfr_id': 'info:sid/{}:generator'.format(rfr_id),
    }
    update_on_value(params, 'rft.title', doc.get('rft.atitle'))
    update_on_value(params, 'rft.date', doc.get('rft.date'))
    update_on_value(params, 'rft.language', doc.get('languages'))

    if doc.get('rft.place') is not None:
        params['rft.place'] = ', '.join(doc.get('rft.place', []))

    genre = doc.get('rft.genre', 'article')
    if genre == 'proceeding':
        genre = 'article'

    if genre == 'book':
        params['rft_val_fmt'] = 'info:ofi/fmt:kev:mtx:book'
        params['rft.genre'] = 'book'
        params['rft.btitle'] = params['rft.title']
        del params['rft.title']

        update_on_value(params, 'rft_id', doc.get('finc.record_id'))
        update_on_value(params, 'rft.btitle', doc.get('rft.btitle'))
        update_on_value(params, 'rft.atitle', doc.get('rft.atitle'))
        update_on_value(params, 'rft.edition', doc.get('rft.edition'))
        update_on_value(params, 'rft.isbn', doc.get('rft.isbn'))
        update_on_value(params, 'rft.issn', doc.get('rft.issn'))
        update_on_value(params, 'rft.eissn', doc.get('rft.eissn'))
        update_on_value(params, 'rft.volume', doc.get('rft.volume'))
        update_on_value(params, 'rft.spage', doc.get('rft.spage'))
        update_on_value(params, 'rft.epage', doc.get('rft.epage'))
        update_on_value(params, 'rft.pages', doc.get('rft.pages'))
        update_on_value(params, 'rft.tpages', doc.get('rft.tpages'))
        update_on_value(params, 'rft.issue', doc.get('rft.issue'))
        update_on_value(params, 'bici', doc.get('bici'))
        update_on_value(params, 'rft.series', doc.get('rft.series'))

        authors = doc.get('authors', [])
        if len(authors) > 0:
            author = authors[0]
            update_on_value(params, 'rft.au', author.get('rft.au'))
            update_on_value(params, 'rft.aucorp', author.get('rft.aucorp'))
            update_on_value(params, 'rft.aufirst', author.get('rft.aufirst'))
            update_on_value(params, 'rft.auinit', author.get('rft.auinit'))
            update_on_value(params, 'rft.auinit1', author.get('rft.auinit1'))
            update_on_value(params, 'rft.aulast', author.get('rft.aulast'))
            update_on_value(params, 'rft.ausuffix', author.get('rft.ausuffix'))

        update_on_value(params, 'rft.genre', doc.get('rft.genre'))
        if doc.get('doi'):
            params['rft_id'] = 'info:doi/{}'.format(doc.get('doi'))

        update_on_value(params, 'rft.pub', doc.get('rft.pub'))

    elif genre == 'article':
        del params['rft.title']

        update_on_value(params, 'rft_id', doc.get('finc.record_id'))
        update_on_value(params, 'rft.atitle', doc.get('rft.atitle'))
        update_on_value(params, 'rft.jtitle', doc.get('rft.jtitle'))
        update_on_value(params, 'rft.stitle', doc.get('rft.stitle'))
        update_on_value(params, 'rft.date', doc.get('rft.date'))
        update_on_value(params, 'rft.issn', doc.get('rft.issn'))
        update_on_value(params, 'rft.eissn', doc.get('rft.eissn'))
        update_on_value(params, 'rft.ssn', doc.get('rft.ssn'))
        update_on_value(params, 'rft.volume', doc.get('rft.volume'))
        update_on_value(params, 'rft.spage', doc.get('rft.spage'))
        update_on_value(params, 'rft.epage', doc.get('rft.epage'))
        update_on_value(params, 'rft.pages', doc.get('rft.pages'))
        update_on_value(params, 'rft.issue', doc.get('rft.issue'))
        update_on_value(params, 'rft.coden', doc.get('rft.coden'))
        update_on_value(params, 'rft.artnum', doc.get('rft.artnum'))
        update_on_value(params, 'sici', doc.get('sici'))
        update_on_value(params, 'rft.chron', doc.get('rft.chron'))
        update_on_value(params, 'rft.quarter', doc.get('rft.quarter'))
        update_on_value(params, 'rft.part', doc.get('rft.part'))

        authors = doc.get('authors', [])
        if len(authors) > 0:
            author = authors[0]
            update_on_value(params, 'rft.au', author.get('rft.au'))
            update_on_value(params, 'rft.aucorp', author.get('rft.aucorp'))
            update_on_value(params, 'rft.aufirst', author.get('rft.aufirst'))
            update_on_value(params, 'rft.auinit', author.get('rft.auinit'))
            update_on_value(params, 'rft.auinit1', author.get('rft.auinit1'))
            update_on_value(params, 'rft.aulast', author.get('rft.aulast'))
            update_on_value(params, 'rft.ausuffix', author.get('rft.ausuffix'))

        update_on_value(params, 'rft.genre', doc.get('rft.genre'))
        if doc.get('doi'):
            params['rft_id'] = 'info:doi/{}'.format(doc.get('doi'))

    elif genre == 'journal':
        update_on_value(params, 'rft.issn', doc.get('rft.issn'))
    else:
        params['rft_val_fmt'] = 'info:ofi/fmt:kev:mtx:book'
        authors = doc.get('authors', [])
        if len(authors) > 0:
            update_on_value(params, 'rft.creator', doc.get('rft.au'))
        update_on_value(params, 'rft.pub', doc.get('rft.pub'))
        update_on_value(params, 'rft.format', doc.get('finc.format'))
        update_on_value(params, 'rft.language', doc.get('languages'))

    return params

def openurl_link_from_intermediateschema(doc, base='http://www.redi-bw.de/links/ubl?rl_site=ubl&',
                                         rfr_id='www.ub.uni-leipzig.de'):
    params = openurl_parameters_from_intermediateschema(doc, rfr_id=rfr_id)
    return '{}{}'.format(base, urlencode(params))



#!/usr/bin/env python
# coding: utf-8

# TODO(miku): use six.
from builtins import *

import io
import logging
import re
import sys

import marcx
import pymarc

copytags = ["005", "007", "008", "020", "024", "041", "042", "084", "093", "100",
            "110", "245", "246", "264", "300", "490", "500", "520", "540",
            "561", "650", "653", "700", "710", "773", "856"]

whitelist = set([
    "1080400",
    "10800",
    "1080401",
    "1080402",
    "1080403",
    "1080404",
    "1080405",
    "1080406",
    "1080407",
    "1080408",
    "1080409",
    "1080410",
    "1080411",
    "1080412",
    "10800",
    "1080400",
    "10899"
])


collection_map = {
    "1080400": u"Massenkommunikation",
    "10800": u"Kommunikationswissenschaften",
    "1080401": u"Rundfunk, Telekommunikation",
    "1080402": u"Druckmedien",
    "1080403": u"Andere Medien",
    "1080404": u"Interaktive, elektronische Medien",
    "1080405": u"Medieninhalte, Aussagenforschung",
    "1080406": u"Kommunikatorforschung, Journalismus",
    "1080407": u"Wirkungsforschung, Rezipientenforschung",
    "1080408": u"Meinungsforschung",
    "1080409": u"Werbung, Public Relations, Öffentlichkeitsarbeit",
    "1080410": u"Medienpädagogik",
    "1080411": u"Medienpolitik, Informationspolitik, Medienrecht",
    "1080412": u"Medienökonomie, Medientechnik",
    "10800": u"Kommunikationswissenschaften",
    "1080400": u"Massenkommunikation (allgemein)",
    "10899": u"Sonstiges zu Kommunikationswissenschaften"
}

def cleaned_subfields(field):
    """
    Given a pymarc.field.Field, return a list of subfields, without empty values.
    """
    cleaned_subfields = []
    it = iter(field.subfields)
    for code, value in zip(it, it):
        if not value:
            continue
        cleaned_subfields.append(code)
        cleaned_subfields.append(value)
    return cleaned_subfields


inputfilename = "30_input.xml"
outputfilename = "30_output.mrc"

if len(sys.argv) >= 3:
    inputfilename, outputfilename = sys.argv[1:3]

inputfile = io.open(inputfilename, "rb")
outputfile = io.open(outputfilename, "wb")

reader = pymarc.parse_xml_to_array(inputfile)

for oldrecord in reader:

    newrecord = marcx.Record()
    newrecord.strict = False

    # prüfen, ob Titel vorhanden ist
    if not oldrecord["245"]:
        continue

    # leader
    newrecord.leader = "     " + oldrecord.leader[5:]
    if len(newrecord.leader) < 9:
        logging.debug("too short %s: %s", len(newrecord.leader), newrecord.leader)
        continue

    # 001
    for identifiers in oldrecord.get_fields("856"):
        for f001 in identifiers.get_subfields("u"):
            regexp = re.search("/document/(\d+)", f001)
            if regexp:
                f001 = regexp.group(1)
                newrecord.add("001", data="finc-30-%s" % f001)
                break
        break

    # Originalfelder, die ohne Änderung übernommen werden
    for tag in copytags:
        for field in oldrecord.get_fields(tag):
            if not field.tag.startswith('00'):
                field.subfields = cleaned_subfields(field)
            newrecord.add_field(field)

    # Reihen / Kollektion
    for collections in oldrecord.get_fields("084"):
        for collection in collections.get_subfields("a"):
            f490a = collection_map.get(collection, "")
            newrecord.add("490", a=f490a)

    # 980
    f980 = ["a", f001, "b", "30", "c", "sid-30-col-ssoar"]
    for collections in oldrecord.get_fields("084"):
        for collection in collections.get_subfields("a"):
            if collection in whitelist:
                f980 = ["a", f001, "b", "30", "c", "sid-30-col-ssoar", "c", "sid-30-col-ssoaradlr"]
                break
    newrecord.add("980", subfields=f980)

    outputfile.write(newrecord.as_marc())

inputfile.close()
outputfile.close()



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
#

"""
VKFilm, #8571. Hebis FTP.

TODO: Unify VK* tasks.

Config
------

[vkfilmff]

ftp-host = host.name
ftp-username = username
ftp-password = password
ftp-path = /
ftp-pattern = *
"""

import datetime

import luigi
from gluish.format import TSV
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.common import FTPMirror
from siskin.task import DefaultTask


class VKFilmFFTask(DefaultTask):
    TAG = '119'

    def closest(self):
        return datetime.date(2016, 9, 26)


class VKFilmFFPaths(VKFilmFFTask):
    """
    Sync.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    max_retries = luigi.IntParameter(default=10, significant=False)
    timeout = luigi.IntParameter(default=20,
                                 significant=False,
                                 description='timeout in seconds')

    def requires(self):
        return FTPMirror(host=self.config.get('vkfilmff', 'ftp-host'),
                         username=self.config.get('vkfilmff', 'ftp-username'),
                         password=self.config.get('vkfilmff', 'ftp-password'),
                         pattern=self.config.get('vkfilmff', 'ftp-pattern'),
                         max_retries=self.max_retries,
                         timeout=self.timeout)

    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class VKFilmFFFincMarc(VKFilmFFTask):
    """
    Find MARC XML, uncompress, clean, remove "Nichtsortierzeichen" on the fly,
    convert via flux (refs #8571).
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return VKFilmFFPaths(date=self.date)

    def run(self):
        with self.input().open() as handle:
            filename = 'film_theater_marc_%s.xml.gz' % (self.closest().strftime("%Y%m%d"))
            for row in handle.iter_tsv(cols=('path',)):

                if not row.path.endswith(filename):
                    continue
                output = shellout("unpigz -c {file} | sed $'s/\u0098//g;s/\u009C//g' > {output}", file=row.path)
                output = shellout("yaz-marcdump -i marcxml -o marc {input} > {output}", input=output)
                output = shellout("flux.sh {flux} inputfile={input} outputfile=stdout > {output}",
                                  flux=self.assets("119/119.flux"), input=output)
                luigi.LocalTarget(output).move(self.output().path)
                break
            else:
                raise RuntimeError('not found: %s' % filename)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
#

"""
VKFilmBerlin, UdK Berlin, #8697.

TODO: Unify VK* tasks.
"""

import datetime

import luigi
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.sources.b3kat import B3KatDownload
from siskin.task import DefaultTask


class VKFilmBerlinTask(DefaultTask):
    TAG = '117'

    def closest(self):
        return monthly(date=self.date)


class VKFilmBerlinRawMARC(VKFilmBerlinTask):
    """
    Next version of filter and transformation, refs #12169. Takes about 4h.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return B3KatDownload(date=self.date)

    def run(self):
        """
        XXX: Nichtsortierzeichen? XXX: cache the number of records, somewhere.
        """
        output = shellout(
            "marccount {input} > {output}", input=self.input().path)
        record_count = int(open(output).read().strip())
        output = shellout("python {script} {input} {output} {record_count}",
                          script=self.assets("117/117_marcbinary.py"),
                          input=self.input().path,
                          record_count=record_count)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))


class VKFilmBerlinMARC(VKFilmBerlinTask):
    """
    Convert binary to XML and remove "Nichtsortierzeichen".
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return VKFilmBerlinRawMARC(date=self.date)

    def run(self):
        """
        XXX: Nichtsortierzeichen? XXX: cache the number of records, somewhere.
        """
        output = shellout("yaz-marcdump -i marc -o marcxml {input} > {output}",
                          input=self.input().path)
        output = shellout("sed 's/\xc2\x98\|\xc2\x9c//g' < {input} > {output}",
                          input=output)
        # MARCXML should be fine as well.
        output = shellout("yaz-marcdump -i marcxml -o marc {input} > {output}",
                          input=output)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



#!/usr/bin/env python3
# coding: utf-8

import io
import re
import sys

import xmltodict
import marcx

inputfilename = "150_input.xml"
outputfilename = "150_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:3]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")

xmlfile = inputfile.read()
xmlrecords = xmltodict.parse(xmlfile)

for xmlrecord in xmlrecords["Records"]["Record"]:

    try:
        f245 = xmlrecord["metadata"]["oai_dc:dc"]["dc:title"]
    except:
        continue

    # Closed Access werden übersprungen
    access = xmlrecord["metadata"]["oai_dc:dc"]["dc:rights"][1]
    if access == "info:eu-repo/semantics/closedAccess":
        continue

    marcrecord = marcx.Record(force_utf8=True)

    # Leader
    marcrecord.leader = "     cam  22        4500"

    # Identifier
    f001 = xmlrecord["header"]["identifier"]
    regexp = re.match("oai:opus.bsz-bw.de-hsmw:(\d+)", f001)
    if regexp:
        f001 = regexp.group(1)
        marcrecord.add("001", data="finc-150-" + f001)

    # 007
    marcrecord.add("007", data="cr")

    # Sprache
    language = xmlrecord["metadata"]["oai_dc:dc"]["dc:language"]
    if language == "deu":
        language = "ger"
    marcrecord.add("008", data="130227uu20uuuuuuxx uuup%s  c" % language)
    marcrecord.add("041", a=language)

    # Verfasser
    try:
        f100a = xmlrecord["metadata"]["oai_dc:dc"]["dc:creator"]
    except:
        continue
    marcrecord.add("100", a=f100a)

    # Titel
    f245 = xmlrecord["metadata"]["oai_dc:dc"]["dc:title"]
    if isinstance(f245, list):
        f245a = f245[0]["#text"]
        f245b = f245[1]["#text"]
        marcrecord.add("245", a=f245a, b=f245b)
    else:
        marcrecord.add("245", a=f245["#text"])

    # Erscheinungsvermerk
    f260c = xmlrecord["metadata"]["oai_dc:dc"]["dc:date"]
    publisher = ["b", "Hochschule Mittweida,", "c", f260c]
    marcrecord.add("260", subfields=publisher)

    # Abstract
    try:
        f520a = xmlrecord["metadata"]["oai_dc:dc"]["dc:description"]["#text"]
        marcrecord.add("520", a=f520a)
    except:
        pass

    # Schlagwörter
    try:
        f689a = xmlrecord["metadata"]["oai_dc:dc"]["dc:subject"]
    except:
        f689a = ""
    if isinstance(f689a, list):
        for subject in f689a:
            if "ddc" not in subject:
                marcrecord.add("689", a=subject)
    else:
        f689a = f689a.split(" , ")
        if len(f689a) > 1:
            for subject in f689a:
                marcrecord.add("689", a=subject)

    # Link zu Datensatz und Ressource
    # Eventuell überprüfen, ob Reihenfolge der Links stimmt "files"
    f856u = xmlrecord["metadata"]["oai_dc:dc"]["dc:identifier"]
    marcrecord.add("856", q="text/html", _3="Link zum Datensatz", u=f856u[0])
    marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u[3])

    # Medientyp
    marcrecord.add("935", b="cofz")
    marcrecord.add("935", c="hs")

    # Kollektion
    marcrecord.add("980", a=f001, b="150", c="sid-150-col-monami")

    outputfile.write(marcrecord.as_marc())


inputfile.close()
outputfile.close()



#!/usr/bin/env python3
# coding: utf-8

# SID: 109
# Collection: Kunsthochschule für Medien Köln (VK Film)
# refs: #8391

"""
Review notes:

* get_datafield depends on a name in module scope, brittle
* get_field_* might be a single function
* format = str(format), seems format can be a list, too
* format, large if/else
"""

from __future__ import print_function

import re
import sys

import marcx
import xmltodict

formatmap = {
    "Buch":
    {
        "leader": "cam",
        "007": "tu",
        "935b": "druck"
    },
    "DVD":
    {
        "leader": "ngm",
        "007": "vd",
        "935b": "dvdv",
        "935c": "vide"
    },
    "Blu-ray":
    {
        "leader": "ngm",
        "007": "vd",
        "935b": "bray",
        "935c": "vide"
    },
    "Videodatei":
    {
        "leader": "cam",
        "007": "cr",
        "935b": "cofz",
        "935c": "vide"
    },
    "CD":
    {
        "leader": "  m",
        "007": "c",
        "935b": "cdda"
    },
    "Videokassette":
    {
        "leader": "cgm",
        "007": "vf",
        "935b": "vika",
        "935c": "vide"
    },
    "Noten":
    {
        "leader": "nom",
        "007": "zm",
        "935c": "muno"
    },
    "Loseblattsammlung":
    {
        "leader": "nai",
        "007": "td",
    },
    "Film":
    {
        "leader": "cam",
        "007": "mu",
        "935b": "sobildtt"
    },
    "Aufsatz":
    {
        "leader": "naa",
        "007": "tu"
    },
    "Objekt":
    {
        "leader": "crm",
        "007": "zz",
        "935b": "gegenst"
    },
    "Zeitschrift":
    {
        "leader": "nas",
        "007": "tu",
        "008": "                     p"
    },
    "Sonstiges":
    {
        "leader": "npa",
        "007": "tu"
    }
}


def get_datafield(tag, code, all=False):
    """
    Return string value for (tag, code) or list of values if all is True.
    """
    values = []
    for field in datafield:
        if field["@tag"] != tag:
            continue
        if isinstance(field["subfield"], list):
            for subfield in field["subfield"]:
                if subfield["@code"] != code:
                    continue
                if not all:
                    return subfield["#text"]
                values.append(subfield["#text"])
        else:
            if field["subfield"]["@code"] == code:
                if not all:
                    return field["subfield"]["#text"]
                values.append(field["subfield"]["#text"])
    return values


def get_leader(format="Buch"):
    if format == "Mehrbänder":
        return "00000cam00000000000a4500"
    return "     %s  22        4500" % formatmap[format]["leader"]


def get_field_007(format="Buch"):
    return formatmap[format]["007"]


def get_field_008(format="Zeitschrift"):
    if "008" not in formatmap[format]:
        return ""
    return formatmap[format]["008"]


def get_field_935b(format="Buch"):
    if "935b" not in formatmap[format]:
        return ""
    return formatmap[format]["935b"]


def get_field_935c(format="Buch"):
    if "935c" not in formatmap[format]:
        return ""
    return formatmap[format]["935c"]


def remove_brackets(field):
    if isinstance(field, list) and len(field) == 0:
        return ""
    return field.replace("<<", "").replace(">>", "")


inputfilename = "109_input.xml"
outputfilename = "109_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
xmlfile = inputfile.read()
records = xmltodict.parse(xmlfile)
outputfile = open(outputfilename, "wb")

parent_title = {}

for record in records:
    
    datafield = record["OAI-PMH"]["ListRecords"]["record"]["metadata"]["record"]["datafield"]

    parent = get_datafield("010", "a")
    if len(parent) > 0:
        parent_title[parent] = ""

for record in records:

    datafield = record["OAI-PMH"]["ListRecords"]["record"]["metadata"]["record"]["datafield"]

    parent = get_datafield("001", "a")
    title = get_datafield("331", "a")

    if parent in parent_title and len(title) > 0:
        parent_title[parent] = title

for i, record in enumerate(records):

    datafield = record["OAI-PMH"]["ListRecords"]["record"]["metadata"]["record"]["datafield"]

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False

    parent = get_datafield("010", "a")
    title = get_datafield("331", "a")

    if "Brockhaus" in title:
        continue

    if len(title) > 0 and len(parent) > 0 and parent in parent_title:
        f245a = parent_title[parent]
        if f245a == "":
            continue
        f245p = title
        f773w = "(DE-576)" + parent
    elif len(title) > 0:
        f245a = title
        f245p = ""
        f773w = ""
    else:
        continue

    # Format
    format = get_datafield("433", "a")
    format = u'%s' % format
    isbn = get_datafield("540", "a")
    isbn = len(isbn)
    parent = get_datafield("010", "a")
    parent = len(parent)
    regexp = re.search("S\.\s\d+\s?-\s?\d+", format)
    if isbn > 0 and "Videokassette" not in format and "VHS" not in format and "DVD" not in format:
        format = "Buch"
    elif ("S." in format or "Bl." in format or "Ill." in format or " p." in format or "XI" in format
                         or "XV" in format or "X," in format or "Bde." in format or ": graph" in format):
        format = "Buch"
    elif "CD" in format:
        format = "CD"
    elif "DVD" in format:
        format = "DVD"
    elif "Blu-ray" in format:
        format = "Blu-ray"
    elif "Videokassette" in format or "VHS" in format or "Min" in format:
        format = "Videokassette"
    elif "Losebl.-Ausg." in format:
        format = "Loseblattsammlung"
    elif regexp:
        format = "Aufsatz"
    elif ("Plakat" in format or "Kassette" in format or "Box" in format or "Karton" in format or "Postkarten" in format
          or "Teile" in format or "USB" in format or "Schachtel" in format or "Schautafel" in format
          or "Medienkombination" in format or "Tafel" in format or "Faltbl" in format or "Schuber" in format):
        format = "Objekt"
    elif parent > 0 and isbn == 0:
        #format = "Zeitschrift"
        continue
    else:
        continue

    # Leader
    f001 = get_datafield("001", "a")
    if f001 in parent_title:
        leader = get_leader(format="Mehrbänder")
    else:
        leader = get_leader(format=format)
    marcrecord.leader = leader

    # Identifier
    marcrecord.add("001", data="finc-109-" + f001)

    # 007
    f007 = get_field_007(format=format)
    marcrecord.add("007", data=f007)

    # 008
    f008 = get_field_008(format=format)
    marcrecord.add("008", data=f008)

    # ISBN
    f020a = get_datafield("540", "a")
    marcrecord.add("020", a=f020a)
    f020a = get_datafield("570", "a")
    marcrecord.add("020", a=f020a)

    # Sprache
    f041a = get_datafield("037", "a")
    marcrecord.add("041", a=f041a)

    # 1. Urheber
    f100a = get_datafield("100", "a")
    f100a = remove_brackets(f100a)
    marcrecord.add("100", a=f100a)

    # Haupttitel & Verantwortlichenangabe
    f245a = remove_brackets(f245a)
    f245c = get_datafield("359", "a")
    f245p = remove_brackets(f245p)
    f245 = ["a", f245a, "c", f245c, "p", f245p]
    marcrecord.add("245", subfields=f245)

    # Erscheinungsvermerk
    f260a = get_datafield("410", "a")
    if isinstance(f260a, list):
        f260a = ""
    f260b = get_datafield("412", "a")
    if isinstance(f260b, list):
        f260b = ""
    f260b = remove_brackets(f260b)
    f260c = get_datafield("425", "a")
    if isinstance(f260c, list):
        f260c = ""
    if f260a != "" and f260b != "":
        f260b = " : " + f260b
    if f260a != "" and f260b == "" and f260c != "":
        f260a = f260a + ", "
    if f260b != "" and f260c != "":
        f260b = f260b + ", "
    f260 = ["a", f260a, "b", f260b, "c", f260c]
    marcrecord.add("260", subfields=f260)

    # Umfangsangabe
    f300a = get_datafield("433", "a")
    f300a = remove_brackets(f300a)
    f300b = get_datafield("434", "a")
    f300 = ["a", f300a, "b", f300b]
    marcrecord.add("300", subfields=f300)

    f490 = get_datafield("451", "a")
    if len(f490) > 0:
        f490 = f490.split(" ; ")
        if len(f490) == 2:
            f490a = f490[0]
            f490v = f490[1]
        else:
            f490a = f490
            f490v = ""
        marcrecord.add("490", a=f490a, v=f490v)

    for f650a in set(get_datafield("710", "a", all=True)):
        f650a = remove_brackets(f650a)
        marcrecord.add("650", a=f650a)

    for f650a in set(get_datafield("711", "a", all=True)):
        f650a = remove_brackets(f650a)
        marcrecord.add("650", a=f650a)

    # weitere Urheber
    for tag in range(101, 200):
        f700a = get_datafield(str(tag), "a")
        f700a = remove_brackets(f700a)
        marcrecord.add("700", a=f700a)

    # weitere Körperschaften
    for tag in range(200, 300):
        f710a = get_datafield(str(tag), "a")
        f710a = remove_brackets(f710a)
        marcrecord.add("710", a=f710a)

    # übergeordnetes Werk
    marcrecord.add("773", w=f773w)

    # Links
    f856u = get_datafield("655", "u")
    f8563 = get_datafield("655", "x")
    if len(f8563) == 0:
        f8563 = u"zusätzliche Informationen"
    if "http" in f856u:
        marcrecord.add("856", q="text/html", _3=f8563, u=f856u)

    # Format
    f935b = get_field_935b(format=format)
    f935c = get_field_935c(format=format)
    marcrecord.add("935", b=f935b, c=f935c)

    # Kollektion
    collection = ["a", f001, "b", "109", "c",
                  u"Kunsthochschule für Medien Köln", "c", "Verbundkatalog Film"]
    marcrecord.add("980", subfields=collection)

    try:
        outputfile.write(marcrecord.as_marc())
    except UnicodeEncodeError as exc:
        print("%s: %s" % (marcrecord["001"], exc), file=sys.stderr)

inputfile.close()
outputfile.close()
